#!/bin/sh
#set -x
#-----------------------------------------------------
# DBNet administration script.
#
# dbnet_admin.sh [start|stop|stop_nowait|restart|status|hup] [quiet]
#
# start, stop, restart, or print status of selected
# DBNet processes, defined in the dbn_admin.tbl
# table.
#
#-----------------------------------------------------

. $HOME/.profile > /dev/null 2>&1

#------- function init --------
#
# Initialize environment
#
#------------------------------

init () {

#
# Local site configuration
#

dbn_home="$DBNROOT"
dbn_bin="$dbn_home/bin"
dbn_tmp="$dbn_home/tmp"
dbn_log="$dbn_home/log"
dbn_table="$dbn_home/table"
log_filename="dbn_admin.log"

me=`whoami`
start_check_level=3 

#
# Environment
#

GREP="/bin/grep"
PS="/bin/ps"
WHOAMI="/bin/whoami"
KILL="/bin/kill"
TEMPFILE_1="$dbn_tmp/$$a"
TEMPFILE_2="$dbn_tmp/$$b"
TEMPFILE_3="$dbn_tmp/$$c"
TEMPFILE_4="$dbn_tmp/$$d"


# Check parameters for ps command
OS=`/bin/uname`

case `/bin/uname` 
in
	Linux)  PS_OPTS="-auxww"
                ;;
        *)      PS_OPTS="-ef"
                ;;
esac


 
}

#----------------function say ------------
#
# if not in quiet mode, echo line to stdout
#
#------------------------------------------

say ( ) {

if [ $1 != "quiet" ]
then
   echo $2
fi;
}


#----------------function test_ps_command -------------
#
# test the functionality of the ps command
# look for this process.
#
# return 0 if working ok, 1 if not.
#
#------------------------------------------------------
test_ps_command () {

this_script=$1

$PS $PS_OPTS > $TEMPFILE_1
status=$?
if [ $status -ne 0 ]
then
   status=1
else
   result=`$GREP -c $this_script $TEMPFILE_1`
   status=$?
   if [ $status -ne 0 ]
   then
      status=1
   else
      if [ $result -eq 0 ]
      then
         status=1
      fi
   fi
fi


[ $status -eq 1 ] && write_to_log loud "ps command is not working!!"

return $status

}


#----------------function test_grep_command -------------
#
# test the functionality of the grep command
#
# return 0 if working ok, 1 if not.
#
#--------------------------------------------------------
test_grep_command () {

cat > $TEMPFILE_1 << END
this
is
a
test
file
END


result=`$GREP -c test $TEMPFILE_1`
status=$?
if [ $status -ne 0 ]
then
   status=1
else
   status=0
fi

[ $status -eq 1 ] && write_to_log $quiet "grep command is not working!!"
return $status

}


#------------------- function start_log ---------------
#
# create dated log file 
#
#------------------------------------------------------

start_log ( ) {

LOG=$dbn_log/$log_filename.`date -u +"%Y%m%d"`
if [ ! -f $LOG ]
then
    touch $LOG
    chmod g+w $LOG
fi
echo  "-----------------------dbn_admin.sh ----------------------" >> $LOG
export LOG;
}

#------------------- function write_to_log ------------------
#
# echo input msg to log file and stdout if not quiet
#
#-----------------------------------------------------------

write_to_log ( ) {
quiet=$1
msg="`date -u +%H%M`:$2"
echo $msg >> $LOG
say $quiet "$2";
}

#--------------- function pid_running --------------------
#
# Check if specified pid is running       
#
# return 0 if running, 1 if not running   
#
#--------------------------------------------------------
pid_running () {

check_pid=$1
running=1

$PS $PS_OPTS > $TEMPFILE_1
status=$?
if [ $status -eq 0 ] 
then
   cat $TEMPFILE_1 | awk '{print $2}' > $TEMPFILE_2
   status=$?
   if [ $status -eq 0 ]
   then
      result=`$GREP -c $check_pid $TEMPFILE_2`
      status=$?
      if [ $status -lt 2 ]
      then
         if [ $result -ne 0 ]
         then
            running=0 
         fi  
      else 
         running=2 
         write_to_log $quiet "Bad status returned from grep: $status"
      fi
   else
      running=2 
      write_to_log $quiet "Bad status returned from awk (1): $status"
   fi
else
   running=2 
   write_to_log $quiet "Bad status returned from ps: $status"
fi

return $running

}
#--------------- function proc_file_running --------------------
#
# Check if specified file is running       
#
# return 0 if running, 1 if not running   
#
#--------------------------------------------------------

proc_file_running () {

proc_file=$1
param_1=$2
param_2=$3
param_3=$4
hup=$5

$PS $PS_OPTS > $TEMPFILE_1
status=$?
if [ $status -eq 0 ]
then
   $GREP -v grep $TEMPFILE_1 > $TEMPFILE_2
   status=$?
   if [ $status -lt 2 ]
   then
      $GREP $me  $TEMPFILE_2 > $TEMPFILE_3
      status=$?
      if [ $status -lt 2 ]
      then
         if [ $param_1 = "none" ]
         then
            proc_string=$proc_file
         else
            if [ $param_2 = "none" ]
            then
               proc_string="$proc_file $param_1"
            else
               if [ $param_3 = "none" ]
               then
                  proc_string="$proc_file $param_1 $param_2"
               else
                  proc_string="$proc_file $param_1 $param_2 $param_3"
               fi
            fi
         fi

         result=`$GREP -c "$proc_string" $TEMPFILE_3`  
         status=$?
         if [ $status -lt 2 ]
         then
            if [ $result -eq 0 ]
            then
               is_running=1
            else
               is_running=0
               if [ $hup = "yes" ] 
               then
                  if [ $result -eq 1 ]     
                  then                       # there is only 1 process found, get it's pid.
                     proc_pid=`$GREP "$proc_string" $TEMPFILE_3 | awk '{print $2}'`
                     kill -HUP $proc_pid
                  else
                     write_to_log $quiet "more than one processes found--can't hup."
                  fi
               fi
            fi
         else
            is_running=2
            write_to_log $quiet "Error returned from grep command (0): $status"
         fi
      else
         is_running=2
         write_to_log $quiet "Error returned from grep command (1): $status"
      fi
   else
      is_running=2
      write_to_log $quiet "Error returned from grep command (2): $status"
   fi
else
   is_running=2
   write_to_log $quiet "Error returned from ps command: $status"
fi

return $is_running

}


#--------------- function proc_running--------------------
#
# Check if specified process is running.
# level 1: Check ps for file name
# level 2: Check ps for pid
# level 3: level 1 & level 2
#
# return 0 if running, 1 if not running
#
#--------------------------------------------------------

proc_running ( )  {

proc_file=$1
param_1=$2
param_2=$3
param_3=$4
pid=$5
is_running=0   # assume it is running

if [ $start_check_level -eq 1 -o $start_check_level -eq 3 ]
then
   proc_file_running $proc_file $param_1 $param_2 $param_3 no
   is_running=$?
fi

if    [ $start_check_level -eq 2 -o \( $start_check_level -eq 3 -a $is_running -eq 1 \) ]
then
   if [ -n "$pid" ]; then
      pid_running $pid
      is_running=$?
   fi
fi

return $is_running;

}


#--------------- function start_dbnet-----------
#
# Start selected dbnet processes
# if not already running.
#
#----------------------------------------------

start_dbnet ( ) {

write_to_log $quiet ""
write_to_log $quiet "Start enabled DBNet processes:"
write_to_log $quiet "-----------------------------"
write_to_log $quiet ""

while read -r proc script_file start_it pid_file hup_it param_1 param_2 param_3
do
   if [ $proc != "#" ]; then
       if [ $start_it = "yes" ]; then
         pid="";
         if [ $pid_file != "none" ]; then
           if [ -a $dbn_tmp/$pid_file ] ; then
              pid=`cat $dbn_tmp/$pid_file`
           fi
         fi
         proc_running $script_file $param_1 $param_2 $param_3 "$pid"  
         start_it=$?
         evaluate_status $start_it $proc "$pid"
         if [ $start_it -eq 1 ]
         then
            write_to_log $quiet "Start $proc"
            cd $dbn_bin
            if [ -a $dbn_tmp/$pid_file ] ; then
                rm $dbn_tmp/$pid_file
            fi
            if [ $param_1 = "none" ]; then
               ./$script_file &
            else
               if [ $param_2 = "none" ]; then
               ./$script_file $param_1 &
               else
                  if [ $param_3 = "none" ]; then
                  ./$script_file $param_1 $param_2 &
                  else
                    ./$script_file $param_1 $param_2 $param_3 &
                  fi
               fi
            fi
         fi
      fi
   fi
done < $dbn_table/dbn_admin.tbl


}

#---------- function wait_until_dead------------
#
# Return when pid is dead, or
# when 20 sec has passed.
# 0 = dead;  1 = not dead
#
#-----------------------------------------------

wait_until_dead ( )  {
pid=$1
is_running=0
wait_limit=20
wait_time=0
still_trying="yes"

while [ $is_running -eq 0 -a -n "$still_trying" ] 
do
   pid_running $pid
   is_running=$?

   if [ $is_running -eq 0 ]
   then
      write_to_log $quiet ".."
      sleep 1
      wait_time=`expr $wait_time + 1`
      [ $wait_time -gt $wait_limit ] && still_trying=""
    fi
done

if [ $is_running -eq 0 ]
then
   return 1
else
   return 0
fi

}


#---------------- function kill_it ----------
#
# Kill the process.  If pid file passed, try
# that.  If not, look for proc file.
#
#--------------------------------------------------------
kill_it ( ) {

wait_for_death=$1
proc_file=$2
pid_file=$3
write_to_log $quiet "Kill $proc_file..."
pid=""
if_running=0

#
# if no pid_file, then look for process to get pid
#

if [ -n $pid_file ]
then 
   [ ! -f $pid_file ] && pid_file=""
fi

if [ -z "$pid_file" ]
then
   $PS $PS_OPTS > $TEMPFILE_1
   status=$?
   if [ $status -eq 0 ]
   then
      $GREP -v grep $TEMPFILE_1 > $TEMPFILE_2
      status=$?
      if [ $status -lt 2 ]
      then
         $GREP $me  $TEMPFILE_2 > $TEMPFILE_3
         status=$?
         if [ $status -lt 2 ]
         then
            $GREP $proc_file $TEMPFILE_3 > $TEMPFILE_4
            status=$?
            if [ $status -lt 2 ]
            then
               pid=`awk '{print $2}' < $TEMPFILE_4`
               if [ -n "$pid" ]
               then
                  is_running=0
               else
                  if_running=1
               fi
               
            else
               is_running=2
               write_to_log $quiet "Error returned from grep command (0): $status"
            fi
         else
            is_running=2
            write_to_log $quiet "Error returned from grep command (1): $status"
         fi
      else
         is_running=2
         write_to_log $quiet "Error returned from grep command (2): $status"
      fi
   else
      is_running=2
      write_to_log $quiet "Error returned from ps command: $status"
   fi
else
   [ ! -f $pid_file ] && pid_file=""
fi

[ -n "$pid_file" ] && pid=`cat $pid_file`
#
# If we've got a pid, kill the process.
# Only wait until it's dead if wait_for_death == 0
#

if [ -n "$pid" ]
then
   $KILL $pid
   if [ $wait_for_death -eq 0 ]
   then
      if wait_until_dead $pid
      then
         write_to_log $quiet "$proc_file process with pid $pid is dead"
      else
         write_to_log $quiet "$proc_file process with pid $pid did not die"
      fi
   else
      write_to_log $quiet "Not waiting for $proc_file process with pid $pid to die..."
   fi
else
   write_to_log $quiet "$proc_file is not running."
fi


}

evaluate_status () {
status=$1
proc=$2
pid=$3
pid_part=""
[ -n "$pid" ] && pid_part=" with pid $pid" 
case $status in 
0) write_to_log $quiet "$proc is running $pid_part" ;;
1) write_to_log $quiet "$proc is not running $pid_part" ;;
*) write_to_log $quiet "Run status of $proc$pid_part is unknown." ;;
esac
}


#---------------- function stop_dbnet -------
#
# Kill all dbnet processes enabled for 
# starting.  
#
#--------------------------------------------
stop_dbnet ( ) {

wait_for_death=$1

write_to_log $quiet ""
write_to_log $quiet "Stop enabled DBNet processes:"
write_to_log $quiet "----------------------------"
write_to_log $quiet ""

while read -r proc script_file start_it pid_file hup_it param_1 param_2 param_3
do
   if [ $proc != "#" ]; then
       if [ $start_it = "yes" ]; then
          if [ $pid_file != "none" ]; then
             kill_it $wait_for_death $script_file $dbn_tmp/$pid_file
          else
             kill_it $wait_for_death $script_file
          fi
      fi
   fi
done < $dbn_table/dbn_admin.tbl

}

#---------------- function display_status  ----------
#
# Display run status of all dbnet processes  
# enabled to start.                 
#
#--------------------------------------------
display_status ( ) {
write_to_log $quiet ""
write_to_log $quiet "Status of enabled DBNet processes:"
write_to_log $quiet "---------------------------------   "
write_to_log $quiet ""

while read -r proc script_file start_it pid_file hup_it param_1 param_2 param_3
do
   if [ $proc != "#" ]; then
       if [ $start_it = "yes" ]; then
         pid=""
         if [ $pid_file != "none" ]; then
           if [ -a $dbn_tmp/$pid_file ] ; then
              pid=`cat $dbn_tmp/$pid_file`
           fi
         fi
         proc_running $script_file $param_1 $param_2 $param_3 "$pid"  
         status=$?
         evaluate_status $status $proc "$pid"
      fi
   fi
done < $dbn_table/dbn_admin.tbl

}


#---------------- function hup_processes  ----------
#
# Send a HUP signal to all enabled processes that
# have are hupable.         
#
#--------------------------------------------
hup_processes  ( ) {

hup_proc=$1;

write_to_log $quiet ""
write_to_log $quiet "HUP the following processes:"
write_to_log $quiet "----------------------------   "

while read -r proc script_file start_it pid_file hup_it param_1 param_2 param_3
do
   if [ $proc != "#" ]; then
       if [ $start_it = "yes" -a $hup_it = "yes" ]; then
          if [ $hup_proc = $proc -o $hup_proc = "all" ]; then
             echo $proc
             proc_file_running $script_file $param_1 $param_2 $param_3 yes
          fi
      fi
   fi
done < $dbn_table/dbn_admin.tbl

}

# ---------------------------------------------------------
#
#                      Main                                
#
# ---------------------------------------------------------

if [ "$2" = "quiet" ]
then
   quiet="quiet"
else
   quiet="loud"
fi

init

this_script=`basename $0`

start_log

# test the grep and ps commands
# if not working properly, exit

test_grep_command 
status=$?
[ $status -ne 0 ] && exit
test_ps_command $this_script
status=$?
[ $status -ne 0 ] && exit

action=$1
if [ -z "$action" ]
then
   say $quiet "usage: `basename $0` start|stop|stop_nowait|restart|status|hup"
   exit
fi

write_to_log $quiet ""
write_to_log $quiet "The following dbnet processes are enabled on this host:"
write_to_log $quiet "------------------------------------------------------ "

while read -r proc script_file start_it pid_file hup_it param_1 param_2 param_3
do
   if [ $proc != "#" ]; then
       [ $start_it = "yes" ] && write_to_log $quiet $proc
   fi
done < $dbn_table/dbn_admin.tbl


case $action in

start|START )
   start_dbnet
   ;;

stop|STOP )
   stop_dbnet 0
   ;;
stop_nowait|STOP_NOWAIT )
   stop_dbnet 1
   ;;

restart|RESTART )
   stop_dbnet 0
   start_dbnet
   ;;

status|STATUS )
   display_status
   ;;

hup|HUP)
   
   if [ -n "$2" ]; then
      hup_processes $2
   else
      hup_processes all
   fi
   ;;



esac

[ -f $TEMPFILE_1 ] &&  rm -f $TEMPFILE_1
[ -f $TEMPFILE_2 ] &&  rm -f $TEMPFILE_2
[ -f $TEMPFILE_3 ] &&  rm -f $TEMPFILE_3
[ -f $TEMPFILE_4 ] &&  rm -f $TEMPFILE_4
