#!/bin/perl

###############################################################################
# This is the DBNet Symbolic Link Maker.
#
# It creates one or more symbolic links to the input filename
# and optionally creates alerts for the new link.   The msgid for the
# file is extracted from the alert filename.  All action for
# a msgid is defined in the slink table, one record for each action.
#
# Format of a slink record:
#
# msgid input_file_regex slink_dir slink_fname [USEGRIBDATE|NOGRIBDATE] [SNDALERT|NOALERT|] {type} {subtype}
# where:
#    msgid 			- is a msgid from the parsing table
#    input_file_regex 		- is used to match the input filename
#    slink_dir 			- is the directory for the symbolic link
#    slink_fname 		- is the filename for the symbolic link
#    [USEGRIBDATE|NOGRIBDATE] 	- denotes where to attempt to extract 
#                         	  the date from the file (in grib format)
#    [SNDALERT|NOALERT|] 	- denotes whether to generate an alert
#    {type} {subtype} 		- are for the generated alert
#
# In slink_dir and slink_fname, allow use of backreferences defined in the
# input_file_regex.  Also allow use of the following date formats, patterned
# after strftime:
#
#    $H = Hour (00-23) 
#    $M = Min  (00-59)
#    $m = Month (01-12)
#    $d = Day of month (01-31)
#    $y = Year without century (00-99)
#    $Y = Year with century (e.g. 2000)
#
# These are evaluted from the current GMT or, if USEGRIBDATE, from the grib file.
#
# usage: dbn_slink.pl <filename> <alert_filename>
#
# alert is of the form ALERT.hostid.msgid...
#        
#
###############################################################################

$dbn_root = $ENV{'DBNROOT'};

if (!defined($dbn_root) || !$dbn_root) {
  print STDERR "\n$0: environment variable DBNROOT not set!\n";
  exit 1;
}

$dbn_config="${dbn_root}/table/dbn_config";
require "$dbn_config" or die "Unable to open $dbn_config";

unshift(@INC, '.');
unshift(@INC, "$include_dir");

require 'dbn_util.pl';

if (!&util'init()) {
  &util'msg($util'ERROR_MSG,"$0: init() failed!");
  exit 1;
}



##########################################################################

require 5.002;             # make sure we're using a new enough version

# set the PATH really early on...

BEGIN { $ENV{PATH} = "/usr/gnu/bin:/usr/bin:/usr/local/bin" }

##########################################################################

#use strict;                # be careful
use Time::CTime;           # for strftime

if (@ARGV < 1) {
   print "usage: dbn_slink.pl filename alert_filename \n";
   exit;
}

if (@ARGV != 2) {
   util'msg($util'ERROR_MSG,"usage: $0: filename alert_filename");
   exit;
}

$orig_fname = shift;     
$alert_fname = shift;
$slink_table = "${dbn_root}/table/slink.tbl";

($alert, $hostid, $msgid) = split(/\./, $alert_fname);

if (length $msgid == 0) {
   printf("no msgid in alert: %s\n", $alert_fname);
   util'msg($util'ERROR_MSG,"$0: no msgid in alert: $alert_fname");
   exit 1;
}


# get the original type and subtype from alert--may need it later.

if (! open (ALERT,"<$alert_fname"))  {
   util'msg($util'ERROR_MSG,"$0: Cannot open alert: $alert_fname");
#  exit 1;
} else {
   $alert_rec = <ALERT>;
   ($source_host, $src_fname, $foo, $foo, $foo, $foo, $orig_alert_type, $job, $orig_alert_subtype) = split(/ /, $alert_rec);
   close (ALERT);
}
   

if (! open (SLINKTABLE,"<$slink_table"))  {
   util'msg($util'ERROR_MSG,"$0: Cannot open slink table: $slink_table");
   exit 1;
}


# read each slink record.  If a msgid rec, then  pattern match orig_filename against orig_file_regex
# and generate new filename, create the directory and symlink the file. 

$found=0;
while (<SLINKTABLE>) {
   ($match_msgid, $orig_file_regex, $new_dir_regex, $new_file_regex, $use_grib_date, 
       $send_alert, $alert_type, $alert_subtype) = split(/\s+/, $_);
   if ( $match_msgid =~ /^#/ ) { next };    # skip comments
   printf ("Try to match %s =~ /%s/\n", $msgid, $match_msgid);
   if ($msgid =~ /^$match_msgid/) {
         printf("found it\n");
      if ($orig_fname =~ /$orig_file_regex/) {
         $found=1;
   
         # Create date variables.  If USEGRIBDATE, then extract from the file.
   
         if ( $use_grib_date =~ /USEGRIBDATE/ ) {
            $gribdate=`$dbn_root/user/gribdate $orig_fname`;
            if ( $? != 0) {
               util'msg($util'ERROR_MSG,"$0: can't execute $dbn_root/user/gribdate $orig_fname: $?");
               exit 1;
            }
            if ( $gribdate !~ /^([\d]+)\ ([\d]+)\ ([\d]+)\ ([\d]+)\ ([\d]+)\ [\d]+\ [\d]+\ [\d]+\ [\d]+\ ([\d]+)/ ) {
              &util'msg($util'ERROR_MSG,"$0: cannot determine gribdate: $gribdate");
              exit 1;
            }
   
            $y=$1; $m=$2; $d=$3; $H=$4; $M=$5; $century=$6;
   
            if ( $y eq "100" ) {
              $y = substr($y, 1, 2);
            }
   
            if ( $century > 20 ) {
              $century = $century - 1;
            }
            $Y = $century . $y;
   
         } else {
            ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) =  gmtime (time);
            $year_with_century =  $year + 1900;
            $year = $year_with_century % 100;
            $mon++;
            $H = sprintf( "%02d",$hour); # HH
            $M = sprintf( "%02d",$min);  # MM
            $m = sprintf( "%02d",$mon);  # MM
            $d = sprintf( "%02d",$mday); # DD
            $y = sprintf( "%02d",$year); # YY
            $Y = sprintf( "%04d",$year_with_century); # YYYY
         }
   
         #printf("Y=%s H=%s M=%s m=%s d=%s y=%s \n", $Y,$H ,$M, $m, $d, $y );
   
   
         # substitute any "%"s for "$" in preparation for date variable substitution.
   
         #$new_file_regex =~ s#%#\$#g;
   
         # define backreference variables from regex.
   
         $orig_fname =~ m#$orig_file_regex#;
   
         # do variable substituion to create the destination filename.
   
         $dest_fname = eval qq/"$new_file_regex"/;
         $dest_dir = eval qq/"$new_dir_regex"/;
         $full_dest_filename = join "/", $dest_dir, $dest_fname;
   
         # create the symlink and issue the dbnet alert.
   
         if (! &util'mkdirs($dest_dir)) {
             &util'msg($util'ERROR_MSG,"$0: cannot create dir $dest_dir");
             exit 1;
         }

         if (! symlink ($orig_fname, $full_dest_filename) ) {                            
            if ( -l $full_dest_filename ) {
               &util'msg($util'ERROR_MSG,"$0: Symlink already exists for $orig_fname");
            } else {
                &util'msg($util'ERROR_MSG,"$0: Cannot create symlink for $full_dest_filename to $orig_fname");
                exit 1;
            }
         } else {
            &util'msg($util'INFO_MSG,"$0: Created symbolic link: $full_dest_filename -> $orig_fname");
         }
 
         if ( $send_alert =~ /SNDALERT/ ) {

         # if no type and subtype are specified, re-alert with the original type 
         # and subtype (obtained from alert file).

            if (($alert_type == "") && ($alert_subtype == "")) {    
               $alert_type = $orig_alert_type;
               $alert_subtype = $orig_alert_subtype;
            }
            system ("$bin_dir/dbn_alert", $alert_type, $alert_subtype, "SLINKER", $full_dest_filename );
         }
      }
   }
}

if ($found == 0) {
   &util'msg($util'INFO_MSG,"$0: No slink record found for $msgid $orig_fname");
}
close (SLINKTABLE);

