/************************************************************************
 *  status.c
 * 
 * status file update for cray conversion. This program will be called
 * to add an entry to the statfile for each product that needs to 
 * be sent to the NWSTG. 
 * command line that includes the jobname, keyword,
 * model and filename.  This method will add a single entry to the stat
 * file.
 *
 **********************************************************************/
#define _POSIX_SOURCE 1

#include <stdio.h>
#include <stdlib.h>
#include "status.h"
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

void PrintUsage(void);
int ValidateKeywordName( char * jn);
int ValidateModelName( char * jn);
int ValidateFileName( char * fn);
int GetSequenceNumber(void);
int UpdateSequenceNumber(int number);
int BeginLock(FILE **fh,struct flock *synclock);
int EndLock(FILE **fh, struct flock *synclock);
int CreateLink(char *key, int seqnumber, char *filename);
  

char *modelstring[NUMMODELNAMES] = {"eta","mso","ngm","avn","mrf","rgl","raw","lfm"};
int modelnumber[NUMMODELNAMES] = {ETA,MSO,NGM,AVN,MRF,RGL,RAW,LFM};
char *keywordstring[NUMKEYWORDNAMES] = {"tran","afos","faxx","grib","xtrn","awip","pasc","fsdr"};
int keywordnumber[NUMKEYWORDNAMES] = {TRAN,AFOS,FAXX,GRIB,XTRN,AWIP,PASC,FSDR};

char *envvar;
char *dbnroot;

int main ( int argc, char *argv[]) 
{
int i=0;
int jobid=-1;
int modelid = -1;
int seqnumber;
FILE *fh=NULL;
struct flock synclock;


  if ((envvar = getenv("STATFILE")) == NULL) {
    fprintf(stderr,"ERROR: ENV var( STATFILE ) not set\n");
    PrintUsage();
    return ENVERROR;
  }

  if ((dbnroot = getenv("DBNROOT")) == NULL) {
    fprintf(stderr,"ERROR: ENV var( DBNROOT ) not set\n");
    PrintUsage();
    return ENVERROR;
  }

  if (argc < MINARGS) {
    fprintf(stderr,"ERROR: Not enough arguments: %d\n",argc);
    PrintUsage();
    return INVALIDARGCOUNT;
  }

  for (i=0;i < strlen(argv[MODELARG]); i++) {
    argv[MODELARG][i] = (char) tolower(argv[MODELARG][i]);
  }

  if ((modelid = ValidateModelName(argv[MODELARG])) == INVALIDARG) {
    fprintf(stderr,"Unknown modelname: %s\n",argv[MODELARG]);
/*  The model name is used for our referance only, report the error but don't care. 
    return INVALIDARG;*/
  }

  for (i=0;i < strlen(argv[KEYWORDARG]); i++) {
    argv[KEYWORDARG][i] = (char) tolower(argv[KEYWORDARG][i]);
  }

  if ((jobid = ValidateKeywordName(argv[KEYWORDARG])) == INVALIDARG) {
    fprintf(stderr,"ERROR: Invaild Keywordname: %s\n",argv[KEYWORDARG]);
    return INVALIDARG;
  }

  if ( ValidateFileName(argv[FILENAMEARG]) == FILENOTFOUND) {
    /*fprintf(stderr,"ERROR: Invaild filename: %s\n",argv[FILENAMEARG]);*/
    return INVALIDARG; 
  }

  if (BeginLock(&fh,&synclock) != 0) {
    return LOCKERROR;
  }


  if ((seqnumber = GetSequenceNumber()) < 0) {
    fprintf(stderr,"ERROR: Unable to update sequence number file\n");
    return FILEERROR;
  } 
  seqnumber = seqnumber >= MAXSEQNUMBER ? 1 : seqnumber+1 ;

  if (UpdateStatusFile(seqnumber,argv[JOBARG],argv[KEYWORDARG],
                       argv[MODELARG],argv[FILENAMEARG])!=GOODRETURN) {
    fprintf(stderr,"ERROR: Unable to update status file\n");
    return FILEERROR;
  }

  if (CreateLink(argv[KEYWORDARG],seqnumber,argv[FILENAMEARG]) != 0) {
    fprintf(stderr,"ERROR: Create Link failed.  Sequence Number not changed.  Update file not current\n");
    return LINKERROR;
  }

  if (UpdateSequenceNumber(seqnumber) != 0) {
    fprintf(stderr,"ERROR: Update failed. update file not current\n");
    return UPDATEERROR;
  }
  
  fprintf(stderr,"Status file Updated: File: %s, added as %4.4d.%4.4s\n",
                 argv[FILENAMEARG],seqnumber,argv[KEYWORDARG]);
  
  if (EndLock(&fh,&synclock) != 0) {
    return LOCKERROR;
  }

  return GOODRETURN;

}


int CreateLink(char *key, int seqnumber, char *filename) 
{
  int i;
  char buf[255];
  char tmp[255];

  if (envvar[strlen(envvar)-1] != '/') 
    sprintf(buf,"%s/%04d",envvar,seqnumber);
  else 
    sprintf(buf,"%s%04d",envvar,seqnumber);

  for (i=0;i<NUMKEYWORDNAMES;i++) {
    sprintf(tmp,"%s.%4s",buf,keywordstring[i]);
    unlink (tmp);
  }

  if (envvar[strlen(envvar)-1] != '/') 
    sprintf(buf,"%s/%04d.%4s",envvar,seqnumber,key);
  else 
    sprintf(buf,"%s%04d.%4s",envvar,seqnumber,key);

  return (symlink(filename,buf));

}

int ValidateFileName(char *fn)
{
  struct stat buf;
  char msg[80];

  if (stat(fn,&buf) == -1) {
    sprintf(msg,"ERROR: File (%s)  stat failed, not added to statfile",fn);
    perror(msg);
    return FILENOTFOUND;
  }

  if (strlen(fn) > MAXFILENAMELEN) {
    fprintf(stderr,"Filename too long, truncated\n");
    return FILENAMETOOLONG;
  }


  return GOODRETURN;
}


int ValidateModelName( char * jn)
{ 
  int i;


  for (i=0;i<NUMMODELNAMES;i++) {
    if (strcmp(jn,modelstring[i]) == 0 ) {
      return modelnumber[i];
    }
  }
  return INVALIDARG;
}

int ValidateKeywordName( char * jn)
{ 
  int i;


  for (i=0;i<NUMKEYWORDNAMES;i++) {
    if (strcmp(jn,keywordstring[i]) == 0 ) {
      return keywordnumber[i];
    }
  }
  return INVALIDARG;
}



int BeginLock(FILE **fh,struct flock *flockptr)
{
  int count = 0; 
  char filename[256];

  strncpy(filename,dbnroot,255);
  if (filename[strlen(filename)-1] != '/') 
    strcat(filename,"/");
  strcat(filename,"tmp/.statlock");

  while ((*fh = fopen(filename,"w")) == NULL ) {
    fprintf(stderr,"Cannot open sync file: %s, in BeginLock, retrying\n",filename);
    sleep(2);
    count++;
    if (count > 5) {
      fprintf(stderr,"ERROR: Open sync file failed\n");
      return -1;
    }
  }

  flockptr->l_type = F_WRLCK;
  flockptr->l_start = 0;
  flockptr->l_whence = SEEK_SET;
  flockptr->l_len = 0;
  count = 0;
  while (fcntl(fileno(*fh),F_SETLK,flockptr) == -1) {
    fprintf(stderr,"Unable to lock sync file, retrying\n");
    sleep(2);
    count++;
    if (count > 5) {
      fprintf(stderr,"ERROR: Lock sync file failed\n");
      fclose(*fh);
      return -1;
    }
  }
  return 0;
}


int EndLock(FILE **fh,struct flock *flockptr)
{
  int count = 0; 
/*
  flockptr->l_type = F_WRLCK;
  flockptr->l_start = 0;
  flockptr->l_whence = SEEK_SET;
  flockptr->l_len = 0;

  if (fcntl(fileno(fh),F_SETLK,flockptr) == -1) {
    fprintf(stderr,"Unable to unlock sync file, retrying\n");
    sleep(2);
    count++;
    if (count > 5) {
      fprintf(stderr,"ERROR: UnLock sync file failed\n");
      fclose(fh);
      return -1;
    }
  }
*/
  fclose(*fh);
  return 0;
}

void PrintUsage(void) {

  fprintf(stdout,"Usage: status keyword model jobname filename\n\n");
  fprintf(stdout, "The ENV variable STATFILE and DBNROOT must also be set.\n");
  fprintf(stdout,"$STATFILE should point to the directory containing the status file.\n");
  fprintf(stdout,"$DBNROOT/tmp will host the lock file.\n");


}

